# -*- coding: utf-8 -*-
# 应用配置
import sys
app={}
app['app_debug']=True  #是否开启调试模式
app['tpl_folder']='./tpl'  #设置模板文件目录名 注意：不能配置目录路径
app['before_request']=''  #设置请求前执行的函数
app['after_request']=''    #设置请求后执行的函数  
app['staticpath']='static'
app['http_server']='wsgiref' #使用的开发服务器 支持 wsgiref
app['cli']=False
app['save_cli_pid']=False #是否开启cli运行时保存pid


#路由配置
route={}
route['default']=True #是否开启默认路由  默认路由开启后面不影响以下配置的路由，模块名/版本名/控制器文件名/方法名 作为路由地址   如：http://www.kcw.com/modular/plug/index/index/
route['modular']='' #指定访问配置固定模块 （如果配置了该值，将无法通过改变url访问不同模块）
route['plug']='' #指定访问固定插件 （如果配置了该值，将无法通过改变url访问不同插件）
route['defmodular']='index' #默认模块 当url不包括模块名时
route['defplug']='index' #默认插件 当url不包括插件名时
route['files']='index' #默认路由文件（控制器） 当url不包括控制器名时
route['funct']='index'  #默认路由函数 (操作方法) 当url不包括操作方法名时
route['methods']=['POST','GET'] #默认请求方式
route['children']=[]

kcw={}
kcw['name']='kcw'                             #项目的名称
kcw['version']='2.6.6'							#项目版本
kcw['description']='超轻量级http框架'       #项目的简单描述
kcw['long_description']='kcw是一个由kcw抽象出来的超轻量级http框架'     #项目详细描述
kcw['license']='MIT License'                    #开源协议   mit开源
kcw['url']='https://docs.kwebapp.cn/index/index/1'
kcw['author']='百里-坤坤'  					 #名字
kcw['author_email']='kcweb@kwebapp.cn' 	     #邮件地址
kcw['maintainer']='坤坤' 						 #维护人员的名字
kcw['maintainer_email']='fk1402936534@qq.com'    #维护人员的邮件地址
kcw['username']=''
kcw['password']=''

domain={}
domain['kcwebfile']="https://file.kwebapp.cn"
domain['kcwebstatic']="https://static.kwebapp.cn"
domain['kcwebimg']="https://img.kwebapp.cn"
domain['kcwebapi']="https://kcwebapi.kwebapp.cn"

#其他配置
other={}

fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr=kcw['name'] #不要修改改参数，否则无法上传模块和插件




# -*- coding: utf-8 -*- 
import os, time, subprocess,psutil
from watchdog.observers import Observer
from watchdog.events import FileSystemEventHandler
class MyFileSystemEventHander(FileSystemEventHandler):
    __eventgtimexz=0
    def __init__(self, fn):
        super(MyFileSystemEventHander, self).__init__()
        self.restart = fn

    def on_any_event(self, event):
        if '.py' in event.src_path and event.src_path.endswith('.py'):
            global eventgtimexz
            if time.time()-self.__eventgtimexz > 0.5:
                self.__eventgtimexz=time.time()
                # print('* 更新文件：%s' % event.src_path,event.event_type)
                if event.event_type=='modified':
                    if 'controller\__init__.py' in event.src_path or 'app\__init__.py' in event.src_path:
                        time.sleep(10)
                        pass
                    else:
                        self.restart()
class Events:
    command = ['echo', 'ok']
    process = None
    def __init__(self,argv):
        # print('event1',argv)
        # if ('--server' not in argv and 'python' not in argv[0]) or 'kcw.py' in argv:
        #     print('event2',argv)
        #     argv.insert(0, 'python')
        self.command = argv
        paths = os.path.abspath('.')
        # print(paths)
        self.start_watch(paths)
    
    def kill_process(self):
        "关闭"
        if self.process:
            if 'kcw'==self.command[0] or 'kcweb'==self.command[0] or 'kcwebplus'==self.command[0]:
                try:
                    process = psutil.Process(self.process.pid)
                except:pass
                else:
                    for proc in process.children(recursive=True):
                        proc.kill()
                        proc.kill()
            else:
                self.process.kill()
                self.process.wait()
            self.process = None
    def start_process(self):
        "启动"
        self.process = subprocess.Popen(self.command)
    def restart_process(self):
        "重启"
        
        self.kill_process()
        time.sleep(0.1)
        self.start_process()

    def start_watch(self,path):
        "执行"
        observer = Observer()
        observer.schedule(MyFileSystemEventHander(self.restart_process), path, recursive=True)
        observer.start()
        self.start_process()
        try:
            while True:
                time.sleep(0.1)
        except KeyboardInterrupt as e:
            self.kill_process()
            # observer.stop()
        # observer.join()
    
# Events(['server.py'])  #执行server.py文件


# -*- coding: utf-8 -*-
import time,hashlib,json,re,os,sys,platform,shutil,requests,importlib,traceback,pip,tarfile,zipfile,signal,psutil,filetype
from kcw import config
from . import globals
class kcwzip:
    def packzip(src,dst):
        "压缩"
        filelist = []
        if os.path.isfile(src):
            filelist.append(src)
        for root, dirs, files in os.walk(src):
            for name in files:
                filelist.append(os.path.join(root, name))
        zf = zipfile.ZipFile(dst, "w", zipfile.zlib.DEFLATED)
        for tar in filelist:
            arcname = tar[len(src):]
            zf.write(tar,arcname)
        zf.close()
    def unzip_file(dst, src):
        "解压"
        zf = zipfile.ZipFile(dst)
        zf.extractall(src)
        zf.close()
class kcwtar:
    def targz(src,dst):
        """
        打包目录为tar.gz
        :param src: 需要打包的目录
        :param dst: 压缩文件名
        :return: bool
        """
        with tarfile.open(dst, "w:gz") as tar:
            tar.add(src, arcname=os.path.basename(src))
        return True
    def untar(dst, src):
        """
        解压tar.gz文件
        :param dst: 压缩文件名
        :param src: 解压后的存放路径
        :return: bool
        """
        try:
            t = tarfile.open(dst)
            t.extractall(path = src)
            return True
        except Exception as e:
            return False

def get_folder():
    '获取当前框架目录'
    return os.path.split(os.path.realpath(__file__))[0][:-7] #当前框架目录
def times():
    """生成时间戳整数 精确到秒(10位数字)
    
    return int类型
    """
    return int(time.time())
def json_decode(strs):
    """json字符串转python类型"""
    try:
        return json.loads(strs)
    except Exception as e:
        if 'JSON object must be str, bytes or bytearray, not list' in str(e):
            return strs
        return []
def json_encode(strs):
    """python列表或字典转成字符串"""
    try:
        return json.dumps(strs,ensure_ascii=False)
    except Exception:
        return ""
def md5(strs):
    """md5加密
    
    参数 strs：要加密的字符串

    return String类型
    """
    m = hashlib.md5()
    b = strs.encode(encoding='utf-8')
    m.update(b)
    return m.hexdigest()
def get_kcweb_cli_pid(route):
    """通过路由地址获取进程号
    
    route 路由地址
    """
    if not os.path.isfile(get_folder()+"/pid/"+md5(route)+"_cli_pid"):
        return False
    pid=False
    with open(get_folder()+"/pid/"+md5(route)+"_cli_pid") as file:
        pid = file.read()
    return pid
def get_kcweb_cli_info(route,types='pid'):
    """通过路由地址获取进程信息
    
    route 路由地址

    types info表示获取进程信息 否则判断进程号是否存在
    """
    pid=get_kcweb_cli_pid(route)
    if pid:
        pid=int(pid)
        try:
            if types=='info':
                p = psutil.Process(pid)
                data={
                    'pid':pid,
                    'name':p.name(),
                    'cli':p.cmdline(),
                    'cpu':p.cpu_percent(1),
                    'memory':p.memory_info().rss
                }
                return data
            else:
                if psutil.pid_exists(pid):
                    return pid
                else:
                    try:
                        os.remove(get_folder()+"/pid/"+md5(route)+"_cli_pid")
                    except:pass
                    return False
        except:
            return False
    else:
        return False
def kill_pid(pid):
    """通过进程结束进程
    
    pid 进程号
    """
    if pid:
        try:
            os.kill(int(pid), signal.SIGTERM)
        except:pass
def kill_route_cli(route):
    """通过路由结束进程
    
    route 路由地址
    """
    pid=get_kcweb_cli_pid(route)
    if pid:
        kill_pid(pid)
        try:
            os.remove(get_folder()+"/pid/"+md5(route)+"_cli_pid")
        except:pass
def save_route_cli_pid(route):
    """通过路由保存进程号(pid)
    
    route 路由地址
    """
    pid = os.getpid()
    f=open(get_folder()+"/pid/"+md5(route)+"_cli_pid",'w')
    f.write(str(pid))
    f.close()
if not os.path.exists(get_folder()+"/pid/"):
    os.makedirs(get_folder()+"/pid/", exist_ok=True)
get_sysinfodesffafew=None
def get_sysinfo():
    """获取系统信息

    return dict类型
    """
    global get_sysinfodesffafew
    if get_sysinfodesffafew:
        sysinfo=get_sysinfodesffafew
    else:
        sysinfo={} 
        sysinfo['platform']=platform.platform()        #获取操作系统名称及版本号，'Linux-3.13.0-46-generic-i686-with-Deepin-2014.2-trusty'  
        sysinfo['version']=platform.version()         #获取操作系统版本号，'#76-Ubuntu SMP Thu Feb 26 18:52:49 UTC 2015'
        sysinfo['architecture']=platform.architecture()    #获取操作系统的位数，('32bit', 'ELF')
        sysinfo['machine']=platform.machine()         #计算机类型，'i686'
        sysinfo['node']=platform.node()            #计算机的网络名称，'XF654'
        sysinfo['processor']=platform.processor()       #计算机处理器信息，''i686'
        sysinfo['uname']=platform.uname()           #包含上面所有的信息汇总，('Linux', 'XF654', '3.13.0-46-generic', '#76-Ubuntu SMP Thu Feb 26 18:52:49 UTC 2015', 'i686', 'i686')
        sysinfo['start_time']=times()
        get_sysinfodesffafew=sysinfo
            # 还可以获得计算机中python的一些信息：
            # import platform
            # platform.python_build()
            # platform.python_compiler()
            # platform.python_branch()
            # platform.python_implementation()
            # platform.python_revision()
            # platform.python_version()
            # platform.python_version_tuple()
    return sysinfo
class create:
    project=''
    appname=None
    modular=None
    path=get_folder() #当前框架目录
    def __init__(self,appname="app",modular="api",project=''):
        self.appname=str(appname)
        self.modular=str(modular)
        if project:
            if os.path.exists(project):
                print('项目已存在，请进入'+str(project)+'目录命令执行命令')
                exit()
            if not os.path.exists(self.appname):
                self.project=str(project)+'/'
                os.makedirs(self.project, exist_ok=True)
    def uninstallplug(self,plug):
        """卸载插件

        plug 插件名
        """
        f=open(self.project+self.appname+"/"+self.modular+"/controller/__init__.py","r",encoding='utf-8')
        text=f.read()
        f.close()
        text=re.sub("\nfrom . import "+plug,"",text)
        text=re.sub("from . import "+plug,"",text)
        f=open(self.project+self.appname+"/"+self.modular+"/controller/__init__.py","w",encoding='utf-8')
        f.write(text)
        f.close()
        shutil.rmtree(self.project+self.appname+"/"+self.modular+"/controller/"+plug)
        return True,"成功"
    def packplug(self,plug):
        """打包插件
        
        plug 插件名
        """
        """打包模块"""
        if os.path.exists(self.project+self.appname+"/"+self.modular+"/controller/"+plug):
            kcwzip.packzip(self.project+self.appname+"/"+self.modular+"/controller/"+plug,self.project+self.appname+"/"+self.modular+"/controller/"+plug+".zip")
            return True,"成功"
        else:
            return False,"失败,插件目录不存在"
    def uploadplug(self,plug,username='',password='',cli=False,relyonlist=[]):
        "上传一个插件"
        if not os.path.isfile(self.project+self.appname+"/"+self.modular+"/controller/"+plug+".zip"):
            self.packplug(plug=plug)
        i=0
        relyonlist=json_encode(relyonlist)
        while True:
            timestamp=times()
            sign=md5(str(username)+str(timestamp)+md5(md5(password)))
            ress=requests.get(config.domain['kcwebapi']+"/user/userinfo/?username="+username+"&timestamp="+str(timestamp)+"&sign="+sign)
            arr=json_decode(ress.text)
            if not arr:
                os.remove(self.project+self.appname+"/"+self.modular+"/controller/"+plug+".zip")
                if config.app['app_debug']:
                    print(ress.text)
                return False,"用户身份验证失败，服务器暂时无法处理"
            if (arr['code']==-1 or arr['code']==2) and cli:
                if i >= 3:
                    os.remove(self.project+self.appname+"/"+self.modular+"/controller/"+plug+".zip")
                    return False,"用户名或密码错误"
                elif i:
                    print("用户名或密码错误，请重新输入")
                    username = input("请输入用户名（手机号）\n")
                    password = input("请输入密码\n")
                else:
                    username = input("请输入用户名（手机号）\n")
                    password = input("请输入密码\n")
                i+=1
            elif arr['code']==0:
                break
            else:
                os.remove(self.project+self.appname+"/"+self.modular+"/controller/"+plug+".zip")
                return False,arr['msg']
        ress=requests.request(url=config.domain['kcwebapi']+"/user/uploadplug/?username="+username+"&timestamp="+str(timestamp)+"&sign="+sign,method='POST',
        data={'name':str(plug),'describes':'','modular':self.modular,'relyonlist':relyonlist,'relyon':config.fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr},
        files={'file':open(self.project+self.appname+"/"+self.modular+"/controller/"+plug+".zip", 'rb')})
        arr=json_decode(ress.text)
        if not arr:
            os.remove(self.project+self.appname+"/"+self.modular+"/controller/"+plug+".zip")
            if config.app['app_debug']:
                print(ress.text)
            return False,"上传失败，服务器暂时无法处理上传"
        elif arr['code']==-1 or arr['code']==2:
            os.remove(self.project+self.appname+"/"+self.modular+"/controller/"+plug+".zip")
            return False,"用户名或密码错误"
        elif arr['code']==0:
            os.remove(self.project+self.appname+"/"+self.modular+"/controller/"+plug+".zip")
            return True,arr['msg']
        elif arr['code']==0:
            os.remove(self.project+self.appname+"/"+self.modular+"/controller/"+plug+".zip")
            return False,arr['msg']
        else:
            os.remove(self.project+self.appname+"/"+self.modular+"/controller/"+plug+".zip")
            return False,arr['msg']
    def installplug(self,plug,edition='',token='',cli=False,mandatory=False,username=''):
        """创建一个插件，如果您的模块目录下没有插件包，则创建默认插件文件
        
        plug 插件名
        """
        plug=str(plug)
        if os.path.exists(self.project+self.appname+"/"+self.modular+"/controller/"+plug) and not mandatory:
            return False,"该插件已存在"
        else:
            i=0
            j=0
            tplug=plug
            modular=self.modular
            while True:
                ress=requests.get(config.domain['kcwebapi']+"/pub/plug",params={"modular":modular,"name":str(tplug),'relyon':config.fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr,"edition":str(edition),"token":token,'username':username})
                arr=json_decode(ress.text)
                # print(modular,tplug,edition,arr)
                if arr:
                    if arr['code']==-1 and cli:
                        if i >= 3:
                            return False,plug+"插件授权码错误"
                        elif i:
                            token = input("授权码错误，请重新输入授权码，从而获得该插件\n")
                        else:
                            token = input("请输入授权码，从而获得该插件\n")
                        i+=1
                    elif arr['code']==-1:
                        return False,plug+"插件授权码错误"
                    elif arr['code']==-5:
                        return False,plug+","+arr['data']
                    elif arr['code']==0 and not arr['data']:
                        modular="api"
                        tplug="index" #默认插件
                        # print(modular,tplug)
                    elif arr['code']==0 and arr['data']:
                        i=0
                        j+=1
                        arr=arr['data']
                        r=requests.get(arr['dowurl'],verify=False)
                        f = open(self.project+self.appname+"/"+self.modular+"/controller/"+plug+".zip", "wb")
                        for chunk in r.iter_content(chunk_size=512):
                            if chunk:
                                f.write(chunk)
                        f.close()
                        if zipfile.is_zipfile(self.project+self.appname+"/"+self.modular+"/controller/"+plug+".zip") and os.path.isfile(self.project+self.appname+"/"+self.modular+"/controller/"+plug+".zip"):
                            break
                        if j >= 10:
                            return False,str(plug)+"插件下载失败"
                        time.sleep(0.1)
                    else:
                        return False,str(plug)+"插件搜索失败"
                else:
                    return False,self.modular+"模块下找不到"+str(plug)+"插件"
            if os.path.isfile(self.project+self.appname+"/"+self.modular+"/controller/"+plug+".zip"):#安装打包好的插件
                kcwzip.unzip_file(self.project+self.appname+"/"+self.modular+"/controller/"+plug+".zip",self.project+self.appname+"/"+self.modular+"/controller/"+plug+"/")
                os.remove(self.project+self.appname+"/"+self.modular+"/controller/"+plug+".zip")
                if os.path.isfile(self.project+self.appname+"/"+self.modular+"/controller/"+plug+"/install.txt"): #安装依赖包
                    install_requires=[]
                    try:
                        f=open(self.project+self.appname+"/"+self.modular+"/controller/"+plug+"/install.txt")
                        while True:
                            line = f.readline()
                            if not line:
                                break
                            elif len(line) > 2:
                                install_requires.append(line)
                        f.close()
                    except:
                        shutil.rmtree(self.project+self.appname+"."+self.modular+"/controller/"+plug)
                        return False,"依赖包错误"
                    if len(install_requires):
                        try:
                            install_requires.insert(0,"install")
                            if 0 != pip.main(install_requires):
                                shutil.rmtree(self.project+self.appname+"/"+self.modular+"/controller/"+plug)
                                return False,"依赖包安装错误"
                        except AttributeError as e:
                            shutil.rmtree(self.project+self.appname+"/"+self.modular+"/controller/"+plug)
                            if config.app['app_debug']:
                                print("建议更新您的pip版本。参考命令：python3 -m pip install --upgrade pip==21.2.4 -i https://mirrors.aliyun.com/pypi/simple/")
                            return False,str(e)
                if os.path.isfile(self.project+self.appname+"."+self.modular+"/controller/"+plug+"/install.py"):
                    try:
                        m=importlib.import_module(self.project+self.appname+"."+self.modular+"/controller/"+plug+".install")
                    except:
                        shutil.rmtree(self.project+self.appname+"."+self.modular+"/controller/"+plug)
                        print(traceback.format_exc())
                        return False,"插件依赖包文件不存在或依赖包文件格式错误"
                    else:
                        try:
                            a=m.install()
                        except:
                            shutil.rmtree(self.project+self.appname+"."+self.modular+"/controller/"+plug)
                            return False,"插件依赖包install函数被破坏"
                        # if not a[0]:
                        #     shutil.rmtree(self.project+self.appname+"."+self.modular+"/controller/"+plug)
                        #     return False,str(a[1])

                f=open(self.project+self.appname+"/"+self.modular+"/controller/__init__.py","r",encoding='utf-8')
                text=f.read()
                f.close()
                text=re.sub("\nfrom . import "+plug,"",text)
                text=re.sub("from . import "+plug,"",text)
                f=open(self.project+self.appname+"/"+self.modular+"/controller/__init__.py","w",encoding='utf-8')
                text+="\nfrom . import "+plug
                f.write(text)
                f.close()

                f=open(self.project+self.appname+"/"+self.modular+"/controller/"+plug+"/common/autoload.py","r",encoding='utf-8')
                text=f.read()
                f.close()
                text=re.sub("app.api",self.appname+"."+self.modular,text)
                f=open(self.project+self.appname+"/"+self.modular+"/controller/"+plug+"/common/autoload.py","w",encoding='utf-8')
                f.write(text)
                f.close()

                
                return True,"插件安装成功，"+plug+"=="+str(arr['edition'])
            else:
                return False,str(plug)+"插件获取失败"
    def uninstallmodular(self):
        "卸载模块"
        f=open(self.project+self.appname+"/__init__.py","r")
        text=f.read()
        f.close()
        text=re.sub("\nfrom . import "+self.modular,"",text)
        text=re.sub("from . import "+self.modular,"",text)
        f=open(self.project+self.appname+"/__init__.py","w")
        f.write(text)
        f.close()
        shutil.rmtree(self.project+self.appname+"/"+self.modular)
        return True,"成功"
    def packmodular(self):
        """打包模块"""
        if self.modular=='index':
            return False,"index为内置模块，因此不能打包"
        if os.path.exists(self.project+self.appname+"/"+self.modular):
            kcwzip.packzip(self.project+self.appname+"/"+self.modular,self.project+self.appname+"/"+self.modular+".zip")
            return True,"成功"
        else:
            return False,"失败，模块目录不存在"
    def uploadmodular(self,username='',password='',cli=False,relyonlist=[]):
        "上传模块"
        if self.modular=='index':
            return False,"index为内置模块，因此不能上传"
        if not os.path.isfile(self.project+self.appname+"/"+self.modular+".zip"):
            self.packmodular()
        i=0
        relyonlist=json_encode(relyonlist)
        while True:
            timestamp=times()
            sign=md5(str(username)+str(timestamp)+md5(md5(password)))
            ress=requests.request(url=config.domain['kcwebapi']+"/user/uploadmodular/?username="+username+"&timestamp="+str(timestamp)+"&sign="+sign,method='POST',
            data={'name':str(self.modular),'describes':'','relyonlist':relyonlist,'relyon':config.fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr},
            files={'file':open(self.project+self.appname+"/"+self.modular+".zip", 'rb')})
            arr=json_decode(ress.text)
            if not arr:
                os.remove(self.project+self.appname+"/"+self.modular+".zip")
                if config.app['app_debug']:
                    print(ress.text)
                return False,"用户身份验证失败，服务器暂时无法处理"
            if (arr['code']==-1 or arr['code']==2) and cli:
                if i >= 3:
                    os.remove(self.project+self.appname+"/"+self.modular+".zip")
                    return False,"用户名或密码错误"
                elif i:
                    print("用户名或密码错误，请重新输入")
                    username = input("请输入用户名（手机号）\n")
                    password = input("请输入密码\n")
                else:
                    username = input("请输入用户名（手机号）\n")
                    password = input("请输入密码\n")
                i+=1
            elif arr['code']==0:
                break
            elif arr['code']==-1:
                os.remove(self.project+self.appname+"/"+self.modular+".zip")
                return False,"用户名或密码错误"
            else:
                os.remove(self.project+self.appname+"/"+self.modular+".zip")
                return False,arr['msg']
        
        ress=requests.request(url=config.domain['kcwebapi']+"/user/uploadmodular/?username="+username+"&timestamp="+str(timestamp)+"&sign="+sign,method='POST',
        data={'name':str(self.modular),'describes':'','relyonlist':relyonlist,'relyon':config.fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr},
        files={'file':open(self.project+self.appname+"/"+self.modular+".zip", 'rb')})
        arr=json_decode(ress.text)
        if not arr:
            os.remove(self.project+self.appname+"/"+self.modular+".zip")
            if config.app['app_debug']:
                print(ress.text)
            return False,"上传失败，服务器暂时无法处理上传"
        elif arr['code']==-1 or arr['code']==2:
            os.remove(self.project+self.appname+"/"+self.modular+".zip")
            return False,"用户名或密码错误"
        elif arr['code']==0:
            os.remove(self.project+self.appname+"/"+self.modular+".zip")
            return True,arr['msg']
        else:
            os.remove(self.project+self.appname+"/"+self.modular+".zip")
            return False,arr['msg']
    def installmodular(self,token='',cli=False):
        "创建模块，如果应用不存，则创建默认应用，如果在您的应用目录下没有模块包，则创建默认模块文件"
        if not os.path.exists(self.project+self.appname):
            if config.fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr=='kcwebplus':
                r=requests.get(config.domain['kcwebfile']+"/kcweb/kcwebplus.zip")
                f = open("./"+self.project+"kcwebplus.zip", "wb")
                for chunk in r.iter_content(chunk_size=512):
                    if chunk:
                        f.write(chunk)
                f.close()
                kcwzip.unzip_file("./"+self.project+"kcwebplus.zip","./"+self.project+self.appname)
                os.remove("./"+self.project+"kcwebplus.zip")
            elif config.fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr=='kcweb':
                r=requests.get(config.domain['kcwebfile']+"/kcweb/kcweb.zip")
                f = open("./"+self.project+"kcweb.zip", "wb")
                for chunk in r.iter_content(chunk_size=512):
                    if chunk:
                        f.write(chunk)
                f.close()
                kcwzip.unzip_file("./"+self.project+"kcweb.zip","./"+self.project+self.appname)
                os.remove("./"+self.project+"kcweb.zip")
            elif config.fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr=='kcw':
                r=requests.get(config.domain['kcwebfile']+"/kcweb/kcw.zip")
                f = open("./"+self.project+"kcw.zip", "wb")
                for chunk in r.iter_content(chunk_size=512):
                    if chunk:
                        f.write(chunk)
                f.close()
                kcwzip.unzip_file("./"+self.project+"kcw.zip","./"+self.project+self.appname)
                os.remove("./"+self.project+"kcw.zip")
            else:
                raise Exception('包名错误')
            if not os.path.isfile("./"+self.project+"server.py"):
                # if "Windows" in platform.platform():
                #     pythonname="python"
                # else:
                #     pythonname="python3.8"
                servertext=('#项目运行文件，请务修改\n'+
                        'import kcw,sys,'+self.appname+'\n'+
                        'app=kcw.web(__name__,'+self.appname+')\n'+
                        'if __name__ == "__main__":\n'+
                        '    try:\n'+
                        '        route=sys.argv[1]\n'+
                        '        if "eventlog"==route:\n'+
                        '            raise Exception("")\n'+
                        '    except:\n'+
                        '        #host监听ip port端口 name python解释器名字 (windows一般是python  linux一般是python3) \n'+
                        '        app.run(host="0.0.0.0",port="39001",name="python3.8")\n'+
                        '    else:\n'+
                        '        app.cli(route)\n'
                        )
                f=open("./"+self.project+"server.py","w+",encoding='utf-8')
                f.write(servertext)
                f.close()
            return True,"应用创建成功"
        else:
            if not os.path.isfile(self.project+self.appname+"/__init__.py") or not os.path.exists(self.project+self.appname+"/common"):
                return False,self.appname+"不是kcw应用"
        if self.modular=='index':
            return False,"index为内置模块，因此不能创建"
        if os.path.exists(self.project+self.appname+"/"+self.modular):
            return False,self.project+self.appname+"/"+self.modular+"已存在"
        else:
            i=0
            modular=self.modular
            while True:
                ress=requests.request(url=config.domain['kcwebapi']+"/pub/modular",method="POST",params={"name":modular,"token":token,'relyon':config.fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr})
                arr=json_decode(ress.text)
                
                if arr:
                    
                    if arr['code']==-1 and cli:
                        if i >= 3:
                            return False,self.modular+"模块授权码错误"
                        elif i:
                            token = input("授权码错误，请重新输入授权码，从而获得该模块\n")
                        else:
                            token = input("请输入授权码，从而获得该模块\n")
                        i+=1
                    elif arr['code']==-1:
                        return False,self.modular+"模块授权码错误"
                    elif not arr['data']:
                        modular="api"
                    elif arr['code']==0 and arr['data']:
                        arr=arr['data']
                        #循环下载模块
                        i=0
                        while i < 5:
                            r=requests.get(arr['dowurl'])
                            f = open(self.project+self.appname+"/"+self.modular+".zip", "wb")
                            for chunk in r.iter_content(chunk_size=1024*100):
                                if chunk:
                                    f.write(chunk)
                            f.close()
                            time.sleep(0.3)
                            if os.path.isfile(self.project+self.appname+"/"+self.modular+".zip"):
                                break
                            i+=1
                        if os.path.isfile(self.project+self.appname+"/"+self.modular+".zip"):#安装打包好的模块
                            kcwzip.unzip_file(self.project+self.appname+"/"+self.modular+".zip",self.project+self.appname+"/"+self.modular+"/")
                            os.remove(self.project+self.appname+"/"+self.modular+".zip")

                            if os.path.isfile(self.project+self.appname+"/"+self.modular+"/install.txt"): #安装依赖包
                                install_requires=[]
                                try:
                                    f=open(self.project+self.appname+"/"+self.modular+"/install.txt")
                                    while True:
                                        line = f.readline()
                                        if not line:
                                            break
                                        elif len(line) > 3:
                                            install_requires.append(line)
                                    f.close()
                                except:
                                    shutil.rmtree(self.project+self.appname+"/"+self.modular)
                                    return False,"模块依赖包错误"
                                if len(install_requires):
                                    try:
                                        install_requires.insert(0,"install")
                                        if 0 != pip.main(install_requires):
                                            shutil.rmtree(self.project+self.appname+"/"+self.modular)
                                            return False,"模块依赖包安装错误"
                                    except AttributeError as e:
                                        shutil.rmtree(self.project+self.appname+"/"+self.modular)
                                        if config.app['app_debug']:
                                            print("建议更新您的pip版本。参考命令：Python -m pip install --user --upgrade pip -i https://mirrors.aliyun.com/pypi/simple/")
                                        return False,str(e)
                            if os.path.isfile(self.project+self.appname+"/"+self.modular+"/install.py"):#如果存在依赖文件
                                try:
                                    m=importlib.import_module(self.project+self.appname+'.'+self.modular+'.install')
                                except:
                                    shutil.rmtree(self.project+self.appname+"/"+self.modular)
                                    print(traceback.format_exc())
                                    return False,"模块依赖包文件不存在或依赖包文件格式错误"
                                else:
                                    try:
                                        a=m.install()
                                    except:
                                        shutil.rmtree(self.project+self.appname+"/"+self.modular)
                                        return False,"模块依赖包install方法被破坏"
                                    # if not a[0]:
                                    #     shutil.rmtree(self.project+self.appname+"/"+self.modular)
                                    #     return False,str(a[1])
                            content="\nfrom . import "+self.modular
                            f=open(self.project+self.appname+"/__init__.py","a",encoding='utf-8')
                            f.write(content)
                            f.close()

                            content=''
                            f=open(self.project+self.appname+"/"+self.modular+"/common/autoload.py","r",encoding='utf-8')
                            while True:
                                line = f.readline()
                                if not line:
                                    break
                                # elif 'from' not in line and 'import' not in line:
                                #     content+=line
                                elif 'from app.common import *' not in line:
                                    content+=line
                            f.close()
                            f=open(self.project+self.appname+"/"+self.modular+"/common/autoload.py","w",encoding='utf-8')
                            f.write("from "+self.appname+".common import *\n"+content)
                            f.close()
                            if os.path.exists(self.project+self.appname+"/"+self.modular+"/controller/index"):
                                content=''
                                f=open(self.project+self.appname+"/"+self.modular+"/controller/index/common/autoload.py","r",encoding='utf-8')
                                while True:
                                    line = f.readline()
                                    if not line:
                                        break
                                    # elif 'from' not in line and 'import' not in line:
                                    #     content+=line
                                    else:
                                        content+=line
                                f.close()
                                f=open(self.project+self.appname+"/"+self.modular+"/controller/index/common/autoload.py","w",encoding='utf-8')
                                f.write("from "+self.appname+"."+self.modular+".common import *\n"+content)
                                f.close()
                        else:
                            return False,self.modular+"模块下载失败"
                        if not os.path.isfile("./server.py"):
                            if "Windows" in platform.platform():
                                pythonname="python"
                            else:
                                pythonname="python3"
                            # sys.argv[0]=re.sub('.py','',sys.argv[0])
                            servertext=('# -*- coding: utf-8 -*-\n#gunicorn -b 0.0.0.0:39010 '+self.appname+':app\n'+
                                    'from kcw import web\n'+
                                    'import '+self.appname+' as application\n'+
                                    'app=web(__name__,application)\n'+
                                    'if __name__ == "__main__":\n'+
                                    '    #host监听ip port端口 name python解释器名字 (windows一般是python  linux一般是python3)\n'+
                                    '    app.run(host="0.0.0.0",port="39001",name="'+pythonname+'")')
                            f=open("./"+self.project+"server.py","w+",encoding='utf-8')
                            f.write(servertext)
                            f.close()
                        return True,"安装成功"
                    else:
                        
                        return False,"模块下载失败"
                else:
                    return False,"找不到"+self.modular+"模块"


from .common import *
import getopt,site
PATH=os.getcwd()
sys.path.append(PATH)
fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr1=''
def get_cmd_par(fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr=''):
    global fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr1
    if fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr:
        fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr1=fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr
    python_version=platform.python_version()
    if python_version[0:3]!='3.8':
        print("\033[1;31;40m "+fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr1+"依赖python3.8，与你现在的python"+python_version+"不兼容")
        exit()
    try:
        opts, args = getopt.getopt(sys.argv[1:], "h", ["project=","app=","modular=","plug=","user=","pwd=","host=","port=","timeout=","processcount=",
        "install","uninstall","pack","upload","cli"])
        # print("opts",opts)
        # print("args",args)
        server=False
        if 'server' in args:
            server=True
        help=False
        if 'help' in args:
            help=True
       
        project=fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr1  #项目名称
        appname='app'  #应用名 目前是固定值 app
        modular='intapp' #模块名
        plug=''  #插件名
        username=''
        password=''
        host='0.0.0.0'
        port=30000
        if fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr1=='kcwebplus':
            port=39001
        timeout='600'
        processcount='4'

        install=False
        uninstall=False
        pack=False
        upload=False
        cli=False
        
        if '--cli' in args:
            cli=True
        i=0
        for data in opts:
            if '--project' == data[0]:
                project=data[1]
            # if '--app' == data[0]:
            #     appname=data[1]
            elif '--modular' == data[0]:
                modular=data[1]
            elif '--plug' == data[0]:
                plug=data[1]
            elif '--user' == data[0]:
                username=data[1]
            elif '--pwd' == data[0]:
                password=data[1]
            elif '--host' == data[0]:
                host=data[1]
            elif '--port' == data[0]:
                port=data[1]
            elif '--timeout' == data[0]:
                timeout=data[1]
            elif '--processcount' == data[0]:
                processcount=data[1]
            
            elif '--help' == data[0]:
                help=True
            elif '--install' == data[0]:
                install=True
            elif '--uninstall' == data[0]:
                uninstall=True
            elif '--pack' == data[0]:
                pack=True
            elif '--upload' == data[0]:
                upload=True
            elif '--cli' == data[0]:
                cli=True
            i+=1
    except Exception as e:
        try:
            gcs=sys.argv[1]
        except:
            gcs=''
        if gcs=='-v':
            if fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr1=='kcw':
                print(config.kcw['name']+"-"+config.kcw['version']) 
            elif fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr1=='kcweb':
                from kcweb.config import kcweb
                print(kcweb['name']+"-"+kcweb['version'])
            elif fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr1=='kcwebplus':
                from kcwebplus.config import kcwebplus
                print(kcwebplus['name']+"-"+kcwebplus['version']) 
        else:
            print("\033[1;31;40m有关"+fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr1+"命令的详细信息，请键入 "+fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr1+" help",e)
        return False
    else:
        return {
            'server':server,
            'project':project,'appname':appname,'modular':modular,'username':username,'password':password,'plug':plug,'host':host,'port':port,'timeout':timeout,'processcount':processcount,
            'help':help,'install':install,'uninstall':uninstall,'pack':pack,'upload':upload,'cli':cli,
            'index':i
        }
def cllfunction():
    global fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr1
    cmd_par=get_cmd_par()
    if not cmd_par:
        exit()
    if cmd_par['help']:
        try:
            cs=sys.argv[2:][0]
        except:
            cs=None
        print("\033[1;31;40m有关某个命令的详细信息，请键入 "+fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr1+" help 命令名")
        print("\033[36m执行 "+fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr1+" help server             可查看server相关命令")
        print("\033[36m执行 "+fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr1+" help modular            可查看赋值相关命令")
        print("\033[36m执行 "+fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr1+" help install            可查看安装相关命令")
        print("\033[36m执行 "+fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr1+" help pack               可查看打包相关命令")
        print("\033[36m执行 "+fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr1+" help upload             可查看上传相关命令")
        print("\033[36m执行 "+fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr1+" help uninstall          可查看卸载相关命令\n")
        if 'server' == cs:
            print("\033[32m"+fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr1+" --host 0.0.0.0 --port 39001 server     启动web服务")
            print("\033[32mhost、port并不是必须的，如果要使用默认值，您可以使用下面简短的命令来启动服务")
            print("\033[32m"+fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr1+" server\n")
        if 'modular' == cs:
            print("\033[32m"+fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr1+" --modular api --plug plug --install    进行安装")
            print("\033[1;31;40m初始化一个web应用示例,通常情况下modular、plug、install同时使用")
            print("\033[32mmodular、plug并不是必须的，如果要使用默认值，您可以使用下面简短的命令来安装")
            print("\033[32m"+fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr1+" install\n")
        if 'install' == cs:
            print("\033[32m"+fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr1+" --install                                                           安装一个默认的应用")
            print("\033[32m"+fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr1+" --modular base --install                                  在app应用中安装一个base模块")
            print("\033[32m"+fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr1+" --modular base --plug plug1 --install                     在app应用base模块中安装一个plug1插件")
            print("\033[32m"+fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr1+" --modular intapp --plug plug1 --user 181*** --install     在app应用intapp模块中安装一个指定用户的plug1插件")
            # print("\033[32m如果您需要新建一个项目 可以使用以下命令")
            # print("\033[32m"+fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr1+" --project projectname --modular intapp --plug plug1 --user 181*** --install     在projectname项目下的app应用下intapp模块中安装一个指定用户的plug1插件\n")
        if 'pack' == cs:
            print("\033[32m"+fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr1+" --modular api --pack                打包一个模块")
            print("\033[32m"+fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr1+" --modular api --plug plug1 --pack   可以打包一个插件\n")
        if 'upload' == cs:
            print("\033[32m"+fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr1+" --modular intapp --user 181*** --pwd pwd123 --upload                上传一个intapp模块")
            print("\033[32m"+fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr1+" --modular intapp --plug plug1 --user 181*** --pwd pwd123 --upload   向intapp模块中上传一个plug1插件")
            print("\033[1;31;40m注意：181*** 和 pwd123 是您的用户或密码")
        if 'uninstall' == cs:
            print("\033[32m"+fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr1+" --modular api --uninstall                  卸载app/api模块")
            print("\033[32m"+fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr1+" --modular api --plug plug1 --uninstall     卸载app/api/plug1插件\n")
    else:
        if cmd_par['cli']:#通过命令行执行控制器的方法
            try:
                obj=importlib.import_module(cmd_par['appname']+'.common')
                if obj.config.fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr!=fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr1:
                    print("该项目只能使用"+obj.config.fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr+"开头的命令")
                    exit()
            except Exception as e:
                print('项目不合法',traceback.format_exc())
                exit()
            config.fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr=fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr1
            from kcw import web
            try:
                import app as application
            except Exception as e:
                if "No module named 'app'" in str(e):
                    print("请在"+fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr1+"项目下运行")
                else:
                    print(traceback.format_exc())
                exit()
            else:
                app=web(__name__,application)
                try:
                    RAW_URI=sys.argv[1]
                except:pass
                else:
                    if RAW_URI=='--cli':
                        RAW_URI=''
                    app.cli(RAW_URI)
        elif cmd_par['server']:#启动web服务
            try:
                obj=importlib.import_module(cmd_par['appname']+'.common')
                if obj.config.fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr!=fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr1:
                    print("该项目只能使用"+obj.config.fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr+"开头的命令")
                    exit()
            except Exception as e:
                print('项目不合法',traceback.format_exc())
                exit()
            config.fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr=fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr1
            types=sys.argv[len(sys.argv)-1]
            if get_sysinfo()['uname'][0]=='Linux':
                pythonpath=site.getsitepackages()[0].replace('\\','/')
                t=pythonpath.split('/')
                tt='/'+t[-3]+'/'+t[-2]+'/'+t[-1]
                pythonpath=pythonpath.replace(tt,'')
                if not os.path.exists('/usr/bin/'+fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr1) and os.path.isfile(pythonpath+'/bin/'+fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr1):
                    os.system("ln -s "+pythonpath+"/bin/"+fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr1+" /usr/bin/"+fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr1)
                # if types=='-stop' or types=='-start':
                #     pass
                # else:
                #     print("启动参数错误，支持 -start和-stop")
                #     exit()
                # try:
                #     f=open("pid",'r')
                #     pid=f.read()
                #     f.close()
                #     if pid:
                #         os.system("kill "+pid)
                # except:pass
                if __name__ == 'kcw.kcw':
                    kill_route_cli('pid/'+fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr1+'_server_pid')
                    if types=='-stop':
                        pass
                    else:
                        save_route_cli_pid('pid/'+fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr1+'_server_pid')
                        from gunicorn.app.wsgiapp import run
                        sys.argv[0] = re.sub(r'(-script\.pyw|\.exe)?$','',sys.argv[0])
                        sys.argv=[sys.argv[0], '-w', str(cmd_par['processcount']), '-b', cmd_par['host']+':'+str(cmd_par['port']),'-t',cmd_par['timeout'], 'server:'+cmd_par['appname']]
                        sys.exit(run())
                        exit()
            else:
                from kcw import web
                try:
                    import app as application
                except Exception as e:
                    if "No module named 'app'" in str(e):
                        print("请在"+fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr1+"项目下运行")
                    else:
                        print(traceback.format_exc())
                    exit()
                else:
                    app=web(__name__,application)
                    if __name__ == 'kcw.kcw':
                        tar=len(sys.argv)
                        kill_route_cli('pid/'+str(sys.argv[tar-1])+fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr1+'_server_pid')
                        if types=='-stop':
                            pass
                        else:
                            save_route_cli_pid('pid/'+str(sys.argv[tar-1])+fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr1+'_server_pid')
                            app.run(host=cmd_par['host'],port=int(cmd_par['port']))
        else:
            if cmd_par['install']:#插入 应用、模块、插件
                if cmd_par['appname'] and cmd_par['modular']:
                    if os.path.exists(cmd_par['appname']):
                        try:
                            obj=importlib.import_module(cmd_par['appname']+'.common')
                            if obj.config.fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr!=fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr1:
                                print("该项目只能使用"+obj.config.fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr+"开头的命令")
                                exit()
                        except Exception as e:
                            print('项目不合法',traceback.format_exc())
                            exit()
                    config.fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr=fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr1
                    server=create(cmd_par['appname'],cmd_par['modular'],project=cmd_par['project'])
                    t=server.installmodular(cli=True)
                    if cmd_par['plug']:
                        t=server.installplug(cmd_par['plug'],cli=True,username=cmd_par['username'])
                        print(t)
                    else:
                        if '应用创建成功' in t[1]:
                            print("创建应用成功，接下来进入入项目目录 在终端中执行："+fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr1+" server 运行项目")
                        else:
                            print(t)
                    return t
                else:
                    print("\033[1;31;40m安装时 必须指定应该app和modular，参考命令： "+fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr1+" --app app --modular api")
                    exit()
            if cmd_par['pack']:#打包 模块、插件
                config.fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr=fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr1
                if cmd_par['appname'] and cmd_par['modular']:
                    server=create(cmd_par['appname'],cmd_par['modular'],project=cmd_par['project'])
                    if cmd_par['plug']:
                        res=server.packplug(plug=cmd_par['plug'])
                    else:
                        res=server.packmodular()
                    print(res)
                    if not res[0]:
                        exit()
                else:
                    print("\033[1;31;40m打包时 必须指定应该app和modular，参考命令： "+fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr1+" --app app --modular api")
                    exit()
            if cmd_par['upload']:#上传 模块、插件
                if cmd_par['appname'] and cmd_par['modular']:
                    try:
                        obj=importlib.import_module(cmd_par['appname']+'.common')
                        if obj.config.fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr!=fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr1:
                            print("该项目只能使用"+obj.config.fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr+"开头的命令")
                            exit()
                    except Exception as e:
                        print('项目不合法',traceback.format_exc())
                        exit()
                    config.fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr=fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr1
                    server=create(cmd_par['appname'],cmd_par['modular'],project=cmd_par['project'])
                    if cmd_par['plug']:
                        res=server.packplug(plug=cmd_par['plug'])
                        if res[0]:
                            res=server.uploadplug(cmd_par['plug'],cmd_par['username'],cmd_par['password'],cli=True)
                        else:
                            print(res)
                            exit()
                    else:
                        res=server.packmodular()
                        if res[0]:
                            res=server.uploadmodular(cmd_par['username'],cmd_par['password'],cli=True)
                        else:
                            print(res)
                            exit()
                    print(res)
                    if not res[0]:
                        exit()
                else:
                    print("\033[1;31;40m上传时 必须指定应该app和modular，参考命令： "+fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr1+" --app app --modular api")
                    exit()
            if cmd_par['uninstall']:#卸载 模块、插件
                config.fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr=fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr1
                if cmd_par['appname'] and cmd_par['modular']:
                    server=create(cmd_par['appname'],cmd_par['modular'],project=cmd_par['project'])
                    if cmd_par['plug']:
                        res=server.uninstallplug(plug=cmd_par['plug'])
                    else:
                        res=server.uninstallmodular()
                    print(res)
                    if not res[0]:
                        exit()
                else:
                    print("\033[1;31;40m卸载时 必须指定应该app和modular，参考命令： "+fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr1+" --app app --modular api")
                    exit()
def cill_start(fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr='kcw'):
    "脚本入口"
    global fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr1
    fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr1=fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr
    return cllfunction()



# 打包上传 python setup.py sdist upload
# 打包并安装 python setup.py sdist install
# twine upload --repository-url https://test.pypi.org/legacy/ dist/* #上传到测试
# pip install --index-url https://pypi.org/simple/ kcweb   #安装测试服务上的kcweb pip3 install kcweb==4.12.4 -i https://pypi.org/simple/
# 安装 python setup.py install
#############################################
import os,sys
from setuptools import setup, find_packages,Extension
kcw={}
kcw['name']='kcw'                             #项目的名称
kcw['version']='2.6.6'							#项目版本
kcw['description']='超轻量级http框架'       #项目的简单描述
kcw['long_description']='kcw是一个由kcw抽象出来的超轻量级http框架'     #项目详细描述
kcw['license']='MIT License'                    #开源协议   mit开源
kcw['url']='https://docs.kwebapp.cn/index/index/1'
kcw['author']='百里-坤坤'  					 #名字
kcw['author_email']='kcweb@kwebapp.cn' 	     #邮件地址
kcw['maintainer']='坤坤' 						 #维护人员的名字
kcw['maintainer_email']='fk1402936534@qq.com'    #维护人员的邮件地址
confkcw={}
confkcw['name']=kcw['name']                            #项目的名称 
confkcw['version']=kcw['version']							#项目版本
confkcw['description']=kcw['description']       #项目的简单描述
confkcw['long_description']=kcw['long_description']     #项目详细描述
confkcw['license']=kcw['license']                   #开源协议   mit开源
confkcw['url']=kcw['url']
confkcw['author']=kcw['author']  					 #名字
confkcw['author_email']=kcw['author_email'] 	     #邮件地址
confkcw['maintainer']=kcw['maintainer'] 						 #维护人员的名字
confkcw['maintainer_email']=kcw['maintainer_email']    #维护人员的邮件地址
def get_file(folder='./',lists=[]):
    lis=os.listdir(folder)
    for files in lis:
        if not os.path.isfile(folder+"/"+files):
            if files=='__pycache__' or files=='.git':
                pass
            else:
                lists.append(folder+"/"+files)
                get_file(folder+"/"+files,lists)
        else:
            pass
    return lists
b=get_file("kcw",['kcw'])
setup(
    name = confkcw["name"],
    version = confkcw["version"],
    keywords = "kcw"+confkcw['version'],
    description = confkcw["description"],
    long_description = confkcw["long_description"],
    license = confkcw["license"],
    author = confkcw["author"],
    author_email = confkcw["author_email"],
    maintainer = confkcw["maintainer"],
    maintainer_email = confkcw["maintainer_email"],
    url=confkcw['url'],
    packages =  b,
    # data_files=[('Scripts', ['kcw/bin/kcw.exe'])],
    install_requires = ['gunicorn==20.0.4','watchdog==4.0.0','filetype==1.2.0','psutil==5.8.0','requests==2.31.0',], #第三方包
    package_data = {
        '': ['*.html', '*.js','*.css','*.jpg','*.png','*.gif'],
    },
    entry_points = {
        'console_scripts':[
            'kcw = kcw.kcw:cill_start'
        ]
    }
)


# -*- coding: utf-8 -*-
import threading,urllib,cgi
from . common import *
from . import config
from datetime import datetime
class web:
    __name=None
    __appname=None
    __config=config 
    __httpd=None
    def __new__(self,name,appname=None):
        self.__name=name
        self.__appname=appname
        if self.__name == '__main__' or self.__name=='kcw.kcw' or self.__name=='kcweb.kcweb' or self.__name=='kcwebplus.kcwebplus':
            # print("web__new__",self.__name)
            return super().__new__(self)
        else:
            def apps(env, start_response):
                p=(config.app['staticpath']+env['RAW_URI'].replace(' ',''))
                status='200 ok'
                if os.path.isfile(p):
                    resheader=[("Cache-Control","public, max-age=2592000"),]
                    ETag=md5(p)
                    try:
                        HTTP_IF_NONE_MATCH=env['HTTP_IF_NONE_MATCH']
                    except KeyError:
                        HTTP_IF_NONE_MATCH=None
                    if HTTP_IF_NONE_MATCH:
                        status="304 Not Modified"
                        body=''
                    else:
                        kind = filetype.guess(p)
                        if kind is None:
                            f=open(p,"rb")
                            body=f.read()
                            f.close()
                        else:
                            f=open(p,"rb")
                            body=f.read()
                            f.close()
                            resheader=[
                                ("Content-Type",kind.mime),
                                ("Cache-Control","public, max-age=43200"),
                                ("Accept-Ranges","bytes"),
                            ]
                            resheader.append(('Content-Type',kind.mime))
                            resheader.append(("Accept-Ranges","bytes"))
                    dateArray = datetime.fromtimestamp(int(time.time())-86400)
                    otherStyleTime = dateArray.strftime('%a, %d %b %Y %H:%M:%S GMT')
                    resheader.append(("Last-Modified",otherStyleTime))
                    resheader.append(("ETag",ETag))
                else:
                    files=None
                    try:
                        CONTENT_TYPE=env['CONTENT_TYPE']
                    except:
                        pass
                    else:
                        if 'multipart/form-data; boundary' in CONTENT_TYPE:#上传文件
                            files = cgi.FieldStorage(fp=env['wsgi.input'], environ=env, keep_blank_values=True)
                        else:
                            files=None
                    env['files']=files
                    try:
                        env['BODY_DATA']=str(env['wsgi.input'].next(), encoding = "utf-8")
                    except:
                        env['BODY_DATA']=""
                    try:
                        env['HTTP_X_REAL_IP']=env['HTTP_X_REAL_IP']
                    except:
                        env['HTTP_X_REAL_IP']=""
                    try:
                        env['url_scheme']=env['HTTP_X_REAL_IP']
                    except:
                        env['url_scheme']=env['wsgi.url_scheme'],
                    status,resheader,body=self.__routes(self,env)
                    if type(body) is bytes:
                        pass
                    else:
                        body=bytes(body, encoding='utf-8')
                # print(env)
                start_response(status,resheader)
                return [body]
            return apps
    # __host=''
    # __port=''
    # __filename=''
    def run(self,host="127.0.0.1",port="39001",name=None):
        """运行开发环境

        host: 监听地址
        
        port: 端口

        name: python命令行解释机名字 (该参数即将废弃)
        """
        # self.__host=host
        # self.__port=port
        
        cmd=sys.argv
        config.app['cli']=False
        if 'eventlog' in cmd or len(cmd)==1 or (len(cmd)>=2 and cmd[len(cmd)-1]=='server'):
            # config.app['cli']=False
            if 'kcwebplus' in cmd[0]: #基于新版本4.13.32之后 kcwebplus server运行
                cmd[0]='kcwebplus'
                filename=cmd[0]
            elif 'kcweb' in cmd[0]: #基于新版本4.13.32之后 kcweb server运行
                cmd[0]='kcweb'
                filename=cmd[0]
            elif 'kcw' in cmd[0]:
                cmd[0]='kcw'
                filename=cmd[0]
            else: #基于老版本4.13.32之前 python server.py运行
                filename=cmd[0][:-3]
            # self.__filename=filename
            if self.__config.app['app_debug']:
                if 'eventlog' in cmd:
                    self.__impl(host=host,port=port,filename=filename)
                else:
                    if name:
                        cmd.insert(0,name)
                    cmd.append('eventlog')
                    from . Events import Events
                    Events(cmd)
            else:
                self.__impl(host=host,port=port,filename=filename)
        elif len(sys.argv)==3 and sys.argv[2]=='--cli':
            try:
                RAW_URI=sys.argv[1]
            except:pass
            else:
                self.cli(RAW_URI)
    def cli(self,RAW_URI):
        config.app['cli']=True
        if RAW_URI:
            PATH_INFO=RAW_URI.split("?")[0] #/aa/bb/cc
            if PATH_INFO[0]=='/':
                PATH_INFO=PATH_INFO[1:]
        else:
            PATH_INFO=RAW_URI
        clipidpath=get_folder()+"/pid/"+md5(RAW_URI)+"_cli_pid"
        pid=''
        if(config.app['save_cli_pid']):
            kill_route_cli(RAW_URI)
            save_route_cli_pid(RAW_URI)
        QUERY_STRING=RAW_URI.replace(str(PATH_INFO),'').replace('?','') #a=1&b=1
        reqheader={ 
            'REQUEST_METHOD':'GET',
            'RAW_URI':RAW_URI,
            'PATH_INFO':PATH_INFO,
            'QUERY_STRING':QUERY_STRING,
            'SERVER_PROTOCOL':'',
            'HTTP_HOST':'',
            'HTTP_COOKIE':'',
            'REMOTE_ADDR':'',
            'HTTP_X_REAL_IP':'',
            'HTTP_USER_AGENT':'',
            'BODY_DATA':''
        }
        status,resheader,body=self.__routes(reqheader)
        print(body)
        if(config.app['save_cli_pid']) and pid:

            try:
                os.remove(clipidpath)
            except:pass
    def __impl(self,host,port,filename):
        "运行测试服务器"
        try:
            # if config.app['http_server']=='wsgiref':
            #     self.__http_wsgi( 
            #         host=host,
            #         port=port,
            #         filename=filename
            #     )
            # elif config.app['http_server']=='kcweb':
            #     self.__http_sever(
            #         host=host,
            #         port=port,
            #         filename=filename
            #     )
            # elif config.app['http_server']=='kcweb_server':
            #     self.__http_server(
            #         host=host,
            #         port=port,
            #         filename=filename
            #     )
            # else:
            #     self.__http_wsgi(
            #         host=host,
            #         port=port,
            #         filename=filename
            #     )
            self.__http_wsgi(
                host=host,
                port=port,
                filename=filename
            )
        except KeyboardInterrupt:
            pass
    
    def __getconfigroute(self,PATH_INFO,header):
        "使用配置路由"
        route=self.__config.route
        routedefault=route['default']
        methods=route['methods']
        paths=''
        for path in PATH_INFO:
            paths+="/"+path
        try:
            for item in route['children']:
                if ':' in item['path']:
                    path=item['path'].split(':')
                    if(len(path)==len(PATH_INFO)):
                        is_pp=False
                        try:
                            item['methods']
                        except:pass
                        else:
                            methods=item['methods']
                        for k in methods: #匹配请求方式
                            if header['REQUEST_METHOD'] in k:
                                is_pp=True
                                break
                        if path[0]==paths[:len(path[0])] and is_pp:
                            del PATH_INFO[0]
                            cs=PATH_INFO
                            PATH_INFO=item['component'].split('/')
                            for v in cs:
                                PATH_INFO.append(v)
                            routedefault=True
                            break
                elif item['path']==paths or item['path']+'/'==paths:
                    PATH_INFO=item['component'].split('/')
                    routedefault=True
                    break
        except:pass
        return routedefault,PATH_INFO
    def __get_modular(self,header):
        "获取模块"
        modular=''
        route=self.__config.route
        if route['modular']:
            if isinstance(route['modular'],str):
                modular=route['modular']
            else:
                HTTP_HOST=header['HTTP_HOST'].split(".")[0]
                for mk in route['modular']:
                    if HTTP_HOST in mk:
                        modular=mk[HTTP_HOST]            
        return modular
    def __get_plug(self,header):
        "获取插件"
        plug=''
        route=self.__config.route
        globals.VAR.defplug=''
        if route['plug']:
            if isinstance(route['plug'],str):
                plug=route['plug']
            else:
                HTTP_HOST=header['HTTP_HOST'].split(".")[0]
                for mk in route['plug']:
                    if HTTP_HOST in mk:
                        plug=mk[HTTP_HOST]
        
        return plug
    def defaultroute(self,header,PATH_INFO):
        "路由匹配"
        globals.VAR.defmodular=''
        globals.VAR.defplug=''
        route=self.__config.route
        modular=web.__get_modular(self,header)
        if route['plug']:
            getplug=web.__get_plug(self,header)
        else:
            getplug=''
        routedefault=route['default']
        methods=route['methods']
        plug=''
        if routedefault:
            files=route['files']
            funct=route['funct']
        else:
            files=''
            funct=''
        param=[]
        urls=''
        i=0
        if modular:
            # if route['plug']:  #匹配模块并且匹配了插件
            #     plug=route['plug']
            if getplug:
                plug=getplug
                routedefault,PATH_INFO=web.__getconfigroute(
                    self,
                    PATH_INFO,
                    header
                )
                if routedefault: #使用路由
                    for path in PATH_INFO:
                        if path:
                            if i==0:
                                files=path
                                urls=urls+"/"+str(path)
                            elif i==1:
                                funct=path
                                urls=urls+"/"+str(path)
                            else:
                                param.append(urllib.parse.unquote(path))
                        i+=1
            else:                  #配置模块没有配置插件
                routedefault,PATH_INFO=web.__getconfigroute(
                    self,
                    PATH_INFO,
                    header
                )
                if routedefault: #使用默认路由
                    for path in PATH_INFO:
                        if path:
                            if i==0:
                                plug=path
                                globals.VAR.defplug=plug
                            elif i==1:
                                files=path
                                urls=urls+"/"+str(path)
                            elif i==2:
                                funct=path
                                urls=urls+"/"+str(path)
                            else:
                                param.append(urllib.parse.unquote(path))
                        i+=1
        # elif route['plug']:  #配置模块但没有匹配插件
        #     plug=route['plug']
        elif getplug:
            # print('getplug',getplug)
            plug=getplug
            routedefault,PATH_INFO=web.__getconfigroute(
                self,
                PATH_INFO,
                header
            )
            if routedefault: #使用默认路由
                for path in PATH_INFO:
                    if path:
                        if i==0:
                            modular=path
                            globals.VAR.defmodular=modular
                        elif i==1:
                            files=path
                            urls=urls+"/"+str(path)
                        elif i==2:
                            funct=path
                            urls=urls+"/"+str(path)
                        else:
                            param.append(urllib.parse.unquote(path))
                    i+=1
        else: #完全默认
            # print('完全默认')
            routedefault,PATH_INFO=web.__getconfigroute(self,PATH_INFO,header)
            if routedefault: #使用默认路由
                for path in PATH_INFO:
                    if path:
                        if i==0:
                            modular=path
                            globals.VAR.defmodular=modular
                        elif i==1:
                            plug=path
                            globals.VAR.defplug=plug
                        elif i==2:
                            files=path
                            urls=urls+"/"+str(path)
                        elif i==3:
                            funct=path
                            urls=urls+"/"+str(path)
                        else:
                            param.append(urllib.parse.unquote(path))
                    i+=1
        #默认路由end ############################################################
        if not modular:
            modular=route['defmodular']
            globals.VAR.defmodular=modular
        if not plug:
            plug=route['defplug']
            globals.VAR.defplug=plug
        return methods,modular,plug,files,funct,tuple(param)
    def __tran(self,data,status,resheader):
        "转换控制器返回的内容"
        if isinstance(data,tuple):
            i=0
            for item in data:
                if i==0:
                    body=item
                elif i==1:
                    status=item
                elif i==2:
                    if isinstance(item,dict):
                        for key in item:
                            resheader[key]=item[key]
                    else:
                        raise Exception('错误！这个不是一个字典')
                else:
                    break
                i+=1
        else:
            body=data
        return body,status,resheader
    def __set_globals(self,header):
        globals.HEADER.GET=header
        # globals.HEADER.url_scheme=header['url_scheme']
        globals.HEADER.Method=header['REQUEST_METHOD']
        globals.HEADER.URL=header['RAW_URI']
        globals.HEADER.PATH_INFO=header['PATH_INFO']
        globals.HEADER.QUERY_STRING=header['QUERY_STRING']
        globals.HEADER.SERVER_PROTOCOL=header['SERVER_PROTOCOL']
        globals.HEADER.HTTP_HOST=header['HTTP_HOST']
        globals.HEADER.BODY_DATA=header['BODY_DATA']
        globals.HEADER.REMOTE_ADDR=header['REMOTE_ADDR']
        globals.HEADER.HTTP_X_REAL_IP=header['HTTP_X_REAL_IP']
        try:
            globals.HEADER.files=header['files']
        except:
            globals.HEADER.files=None
        try:
            globals.HEADER.HTTP_COOKIE=header['HTTP_COOKIE']
        except:
            globals.HEADER.HTTP_COOKIE=None
        try:
            globals.HEADER.HTTP_USER_AGENT=header['HTTP_USER_AGENT']
        except:
            globals.HEADER.HTTP_USER_AGENT=None
    def __del_globals():
        globals.VAR = threading.local()
        globals.HEADER = threading.local()
        globals.G = threading.local()
    def __routes(self,header):
        body="这是一个http测试服务器"
        status="200 ok"
        resheader={"Content-Type":"application/json; charset=utf-8"}
        web.__set_globals(self,header)
        PATH_INFO=header['PATH_INFO'].split('/')
        if PATH_INFO[0]==' ' or PATH_INFO[0]=='':
            del PATH_INFO[0]
        methods,modular,plug,files,funct,param=web.defaultroute(self,header,PATH_INFO)
        globals.VAR.component=(methods,modular,plug,files,funct,param)
        if header['REQUEST_METHOD'] in methods:
            try:
                obj=getattr(web.__appname,modular)
            except (AttributeError,UnboundLocalError):
                status="500 Internal Server Error"
                body=web.__tpl(
                    title = status,
                    e=status,
                    data="路由不存在："+str(modular)+"/"
                )
            else:
                try:
                    obj=getattr(obj,"controller")
                except (AttributeError,UnboundLocalError):
                    status="404 Not Found"
                    body=web.__tpl(
                        title = status,
                        e=status,
                        data="路由不存在："+str(modular)+"/controller/"
                    )
                else:
                    try:
                        obj=getattr(obj,plug)
                    except (AttributeError,UnboundLocalError) as e:
                        con="无法找到目录："+str(modular)+"/controller/"+str(plug)+"/"
                        try:
                            data=getattr(obj,"error")(e,con)
                            body,status,resheader=web.__tran(
                                self,
                                data,
                                status,
                                resheader
                            )
                        except (AttributeError,UnboundLocalError):
                            status="404 Not Found"
                            body=web.__tpl(
                                title = status,
                                e=status,data=con
                            )
                        except Exception as e:
                            status="500 Internal Server Error"
                            errms=status
                            if self.__config.app['app_debug']:
                                print(traceback.format_exc())
                                errms=traceback.format_exc().split("\n")
                            body=web.__tpl(
                                title = status,
                                data=errms,e=e
                            )
                    else:
                        try:
                            obj=getattr(obj,files)
                        except (AttributeError,UnboundLocalError) as e:
                            con="无法找到文件："+str(modular)+"/controller/"+str(plug)+"/"+str(files)+".py"
                            try:
                                data=getattr(obj,"error")(e,con)
                                body,status,resheader=web.__tran(
                                    self
                                    ,data
                                    ,status
                                    ,resheader
                                )
                            except (AttributeError,UnboundLocalError):
                                status="404 Not Found"
                                body=web.__tpl(
                                    title = status
                                    ,data=con
                                    ,e=status)
                            except Exception as e:
                                status="500 Internal Server Error"
                                errms=status
                                if self.__config.app['app_debug']:
                                    print(traceback.format_exc())
                                    errms=traceback.format_exc().split("\n")
                                body=web.__tpl(
                                    title = status,
                                    data=errms,
                                    e=e
                                )
                        else:
                            try:
                                data=None
                                if self.__config.app['before_request']:  #请求前执行的函数
                                    try:
                                        data=getattr(obj,self.__config.app['before_request'])()
                                        if data:
                                            body,status,resheader=web.__tran(
                                                self,data,
                                                status,
                                                resheader
                                            )
                                    except (AttributeError):
                                        print(traceback.format_exc())
                                        pass
                                    except Exception as e:
                                        try:
                                            data=getattr(obj,"error")(e,traceback.format_exc().split("\n"))
                                            body,status,resheader=web.__tran(
                                                self,data,
                                                status,
                                                resheader
                                            )
                                        except (AttributeError):
                                            data=True
                                            status="500 Internal Server Error"
                                            errms=status
                                            if self.__config.app['app_debug']:
                                                # print(traceback.format_exc())
                                                errms=traceback.format_exc().split("\n")
                                            body=web.__tpl(
                                                title = status,
                                                data=errms,e=e
                                            )
                                        except Exception as e:
                                            data=True
                                            status="500 Internal Server Error"
                                            errms=status
                                            if self.__config.app['app_debug']:
                                                print(traceback.format_exc())
                                                errms=traceback.format_exc().split("\n")
                                            body=web.__tpl(
                                                title = status,
                                                data=errms,e=e
                                            )
                                if not data:
                                    try:
                                        objclass=getattr(obj,files) #新增
                                    except:
                                        data=getattr(obj,funct)(*param) #兼容性之前的控制器
                                    else:
                                        data=getattr(objclass,funct)(*param)
                                    body,status,resheader=web.__tran(
                                        self,data,
                                        status,
                                        resheader
                                    )
                            except Exception as e:
                                try:
                                    data=getattr(obj,"error")(e,traceback.format_exc().split("\n"))
                                    body,status,resheader=web.__tran(
                                        self,data,
                                        status,
                                        resheader
                                    )
                                except (AttributeError):
                                    status="500 Internal Server Error"
                                    errms=status
                                    if self.__config.app['app_debug']:
                                        print(traceback.format_exc())
                                        errms=traceback.format_exc().split("\n")
                                    body=web.__tpl(
                                        title = status,
                                        data=errms,
                                        e=e
                                    )
                                except Exception as e:
                                    status="500 Internal Server Error"
                                    errms=status
                                    if self.__config.app['app_debug']:
                                        print(traceback.format_exc())
                                        errms=traceback.format_exc().split("\n")
                                    body=web.__tpl(
                                        title = status,
                                        data=errms,
                                        e=e
                                    )
        else:
            status="405 Method Not Allowed"
            body=web.__tpl(
                title = status,
                data='405 Method Not Allowed',
                e=''
            )
        try:
            resheader['set-cookie']=globals.set_cookie
            del globals.set_cookie
        except:pass
        
        if self.__config.app['after_request']:  #请求后执行的函数
            try:
                data=getattr(obj,self.__config.app['after_request'])(body,status,resheader)
                if data:
                    body,status,resheader=web.__tran(self,data,status,resheader)
            except (AttributeError,UnboundLocalError):pass
            except Exception as e:
                try:
                    data=getattr(obj,"error")(e,traceback.format_exc().split("\n"))
                    body,status,resheader=web.__tran(
                        self,data,
                        status,
                        resheader
                    )
                except AttributeError as e:
                    status="500 Internal Server Error"
                    errms=status
                    if self.__config.app['app_debug']:
                        print(traceback.format_exc())
                        errms=traceback.format_exc().split("\n")
                    body=web.__tpl(
                        title = status
                        ,data=errms,
                        e=e
                    )
                except Exception as e:
                    status="500 Internal Server Error"
                    errms=status
                    if self.__config.app['app_debug']:
                        print(traceback.format_exc())
                        errms=traceback.format_exc().split("\n")
                    body=web.__tpl(
                        title = status,
                        data=errms,
                        e=""
                    )
        resheaders=[]
        for key in resheader:
            resheaders.append((key,resheader[key]))
        web.__del_globals()

        
        if self.__config.fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr=='kcweb' or self.__config.fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr=='kcwebplus':#请求后执行的函数
            try:
                getattr(obj,"rfdsgeagesegdsfdsfdrrsebskcwebafterrequest")() #关闭数据库相关连接
            except:pass


        if isinstance(resheaders,list):
            if not body:
                body=''
            if type(body) is bytes:
                pass
            else:
                body=str(body)
            return str(status),resheaders,body
        else:
            raise Exception()
    def __tpl(**context):
        arr=json_encode({
            'title':context['title'],
            'msg':str(context['e']),
            'error':context['data']
        })
        return arr
    # def __tpl(**context):
    #     title,status,error=context
    #     arr=json_encode({
    #         'title':title,
    #         'status':status,
    #         'error':error
    #     })
    #     headers={"Content-Type":"application/json; charset=utf-8","Access-Control-Allow-Origin":"*"}
    #     return arr,'500 error',headers
    def __application(self,env, start_response):
        # print(env)
        try:
            request_body_size = int(env.get('CONTENT_LENGTH', '0'))
        except (ValueError):
            request_body_size = 0
        if 'multipart/form-data; boundary' in env['CONTENT_TYPE']:#上传文件
            files = cgi.FieldStorage(fp=env['wsgi.input'], environ=env, keep_blank_values=True)
            # open(form["file"].filename, 'wb').write(form["file"].value) 
            # print(files["file"].name)
            # print(files["name"].filename)
            # print(files["name"].value)
            env['BODY_DATA']=None
        else:
            files=None
            BODY_DATA=env['wsgi.input'].read(request_body_size)
            env['BODY_DATA']=BODY_DATA.decode()
        env['RAW_URI']=env['PATH_INFO']
        if env.get('QUERY_STRING'):
            env['RAW_URI']=str(env['PATH_INFO'])+"?"+str(env.get('QUERY_STRING'))
        try:
            env['REMOTE_ADDR']
        except:
            env['REMOTE_ADDR']=''
        try:
            env['wsgi.url_scheme']
        except:
            env['wsgi.url_scheme']=''
        try:
            env['HTTP_X_REAL_IP']
        except:
            env['HTTP_X_REAL_IP']=''
        try:
            env['HTTP_COOKIE']
        except:
            env['HTTP_COOKIE']=''
        try:
            env['HTTP_USER_AGENT']
        except:
            env['HTTP_USER_AGENT']=''
        reqheader={ 
            'REQUEST_METHOD':env['REQUEST_METHOD'],
            'url_scheme':env['wsgi.url_scheme'],
            'RAW_URI':env['RAW_URI'],
            'PATH_INFO':env['PATH_INFO'],
            'QUERY_STRING':env['QUERY_STRING'],
            'SERVER_PROTOCOL':env['SERVER_PROTOCOL'],
            'HTTP_HOST':env['HTTP_HOST'],
            'HTTP_COOKIE':env['HTTP_COOKIE'],
            'REMOTE_ADDR':env['REMOTE_ADDR'],
            'HTTP_USER_AGENT':env['HTTP_USER_AGENT'],
            'BODY_DATA':env['BODY_DATA'],
            'HTTP_X_REAL_IP':env['HTTP_X_REAL_IP'],
            'files':files
        }
        p=(config.app['staticpath']+reqheader['RAW_URI'].replace(' ',''))
        status='200 ok'
        if os.path.isfile(p):
            resheader=[("Cache-Control","public, max-age=2592000"),("Accept-Ranges","bytes")]
            ETag=md5(p)
            try:
                HTTP_IF_NONE_MATCH=globals.HEADER.GET['HTTP_IF_NONE_MATCH']
            except:
                HTTP_IF_NONE_MATCH=None
            if HTTP_IF_NONE_MATCH:
                status="304 Not Modified"
            else:
                kind = filetype.guess(p)
                if kind is None:
                    f=open(p,"rb")
                    body=f.read()
                    f.close()
                    if '.css' in p[-5:]:
                        resheader.append(("Content-Type","text/css"))
                    elif '.js' in p[-5:]:
                        resheader.append(("Content-Type","application/javascript"))
                else:
                    f=open(p,"rb")
                    body=f.read()
                    f.close()
                    resheader.append(("Content-Type",kind.mime))
            dateArray = datetime.fromtimestamp(int(time.time())-86400)
            otherStyleTime = dateArray.strftime('%a, %d %b %Y %H:%M:%S GMT')
            resheader.append(("Last-Modified",otherStyleTime))
            resheader.append(("ETag",ETag))
        else:
            status,resheader,body=web.__routes(self,reqheader)
            resheader.append(('Server', "kcw"+config.kcw['version']))
            if type(body) is bytes:
                pass
            else:
                body=bytes(body, encoding='utf-8')
        start_response(status,resheader)
        return [body]
    
    def __http_wsgi(self,host,port,filename):
        "http——wsgi测试服务"
        from wsgiref.simple_server import make_server
        print("* \033[1;31;40m! 警告：\033[0m这是一个wsgiref开发服务器。不要在生产环境中部署使用它")
        # print('* 生产环境中建议使用gunicorn,gunicorn运行命令如：gunicorn -b '+host+':'+str(port)+' '+str(filename)+':app')
        if host=='0.0.0.0':
            print("\033[32m* （CTRL+单击打开）http://127.0.0.1:"+str(port)+"（按CTRL+C退出）")
        else:
            print("\033[32m* 运行在http://"+host+":"+str(port)+"（按CTRL+C退出）")
        if self.__config.app['app_debug']:
            print('\033[37m* 调试器：开启')
        else:
            print('\033[37m* 调试器：已关闭')
        self.__httpd = make_server(host, int(port), self.__application)
        self.__httpd.serve_forever()
    # def __server_client(self,new_socket):
    #     # 处理http的的请求
    #     data=new_socket.recv(1047576).decode()
    #     if data:
    #         datas=data.split("\r\n")
    #         data1=datas[0]
    #         #reqsest
    #         REQUEST_METHOD=data1.split("/")[0].replace(' ','') ##GET
    #         RAW_URI=re.findall(REQUEST_METHOD+"(.+?) HTTP", data1) #/aa/bb/cc?a=1&b=1
    #         if RAW_URI:
    #             RAW_URI=RAW_URI[0]
    #         else:
    #             RAW_URI=''
    #         PATH_INFO=RAW_URI.split("?")[0] #/aa/bb/cc
    #         QUERY_STRING=RAW_URI.replace(str(PATH_INFO),'').replace('?','') #a=1&b=1
    #         SERVER_PROTOCOL=data1.split(" ")[-1] #HTTP/1.1
    #         HTTP_HOST=re.findall("Host: (.+?)\r\n", data)#212.129.149.238:39010
    #         if HTTP_HOST:
    #             HTTP_HOST=HTTP_HOST[0]
    #         else:
    #             HTTP_HOST=''
    #         HTTP_COOKIE=re.findall("Cookie: (.+?)\r\n", data)#cookie
    #         if HTTP_COOKIE:
    #             HTTP_COOKIE=HTTP_COOKIE[0]
    #         else:
    #             HTTP_COOKIE=''
    #         REMOTE_ADDR=''
    #         HTTP_USER_AGENT=re.findall("User-Agent: (.+?)\r\n", data) #Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.36 SE 2.X MetaSr 1.0
    #         if HTTP_USER_AGENT:
    #             HTTP_USER_AGENT=HTTP_USER_AGENT[0]
    #         else:
    #             HTTP_USER_AGENT=''
    #         BODY_DATA=datas[len(datas)-1]
    #         # print(data)
    #         #reqsest
    #         reqheader={ 
    #             'REQUEST_METHOD':REQUEST_METHOD,
    #             'RAW_URI':RAW_URI,
    #             'PATH_INFO':PATH_INFO,
    #             'QUERY_STRING':QUERY_STRING,
    #             'SERVER_PROTOCOL':SERVER_PROTOCOL,
    #             'HTTP_HOST':HTTP_HOST,
    #             'HTTP_COOKIE':HTTP_COOKIE,
    #             'REMOTE_ADDR':REMOTE_ADDR,
    #             'HTTP_X_REAL_IP':'',
    #             'HTTP_USER_AGENT':HTTP_USER_AGENT,
    #             'BODY_DATA':BODY_DATA
    #         }
    #         p=(config.app['staticpath']+RAW_URI.replace(' ',''))
    #         # print("目录",p)
    #         status='200 ok'
    #         if os.path.isfile(p):
    #             # print('静态文件',p)
    #             kind = filetype.guess(p)
    #             if kind is None:
                    
    #                 f=open(p,"rb")
    #                 body=f.read()
    #                 f.close()
    #                 resheader=[("Cache-Control","public, max-age=43200"),("Expires","Thu, 07 Nov 2019 02:59:02 GMT")]
                    
    #                 header="HTTP/1.1 %s \n" % status
    #                 header+="Content-Length:%d\n" % len(body)
    #             else:
    #                 f=open(p,"rb")
    #                 body=f.read()
    #                 f.close()
    #                 resheader=[("Content-Type",kind.mime),("Cache-Control","public, max-age=43200"),("Accept-Ranges","bytes"),("Expires","Thu, 07 Nov 2019 02:59:02 GMT")]
    #                 header="HTTP/1.1 %s \n" % status
    #                 header+="Content-Length:%d\n" % len(body)
    #         else:
    #             status,resheader,body=self.__routes(reqheader)
    #             if type(body) is bytes:
    #                 pass
    #             else:
    #                 body=body.encode()
    #             header="HTTP/1.1 %s \n" % status
    #             header+="Content-Length:%d\n" % len(body)
            
    #         print(HTTP_HOST+' -- ['+str(time.strftime("%Y-%m-%d %H:%M:%S", time.localtime()))+'] "'+REQUEST_METHOD+" "+RAW_URI +" "+SERVER_PROTOCOL + '" '+status+"-")
    #         t=time.time()
    #         header+="Server:kcweb"+str(config.kcweb['version'])+"\n"
    #         header+="Date:%s\n" % datetime.utcnow().strftime('%a, %d %b %Y %H:%M:%S GMT')
    #         for t in resheader:
    #             header+="%s:%s\n" % (t[0],t[1])
    #         header+="\n"
    #         try:
    #             new_socket.send(header.encode())
    #             new_socket.send(body)
    #         except Exception as e:
    #             pass
    #     new_socket.close() 
    # def __http_server(self,host,port,filename):
    #     "http测试服务"
    #     tcp_socket=socket.socket(socket.AF_INET,socket.SOCK_STREAM)
    #     try:
    #         tcp_socket.bind((host,int(port)))
    #     except OSError:
    #         print("通常每个套接字地址(协议/网络地址/端口)只允许使用一次（按CTRL+C退出）")
    #     else:
    #         tcp_socket.listen(1024)
    #         print('! \033[1;31;40m警告：\033[0m这是一个kcweb_server开发服务器。不要在生产环境中部署使用它')
    #         print('* 生产环境中建议使用gunicorn,gunicorn运行命令如：gunicorn -b '+host+':'+str(port)+' '+str(filename)+':app')
    #         if self.__config.app['app_debug']:
    #             print('* 调试器：开启')
    #         else:
    #             print('* 调试器：已关闭')
    #         print("* 运行在http://"+host+":"+str(port)+"/ （按CTRL+C退出）")
    #         while True:
    #             new_tcp_socket,client_info=tcp_socket.accept()
    #             t=threading.Thread(target=self.__server_client,args=(new_tcp_socket,))
    #             t.daemon=True
    #             t.start()
    #         tcp_socket.close()
    
    # def __http_sever(self,host,port,filename):
    #     #http测试服务器
    #     print('\033[1;31;40m! 警告：\033[0m这是一个kcweb开发服务器。不要在生产环境中部署使用它')
    #     print('* 生产环境中建议使用gunicorn,gunicorn运行命令如：gunicorn -b '+host+':'+str(port)+' '+str(filename)+':app')
    #     if self.__config.app['app_debug']:
    #         print('* 调试器：开启')
    #     else:
    #         print('* 调试器：已关闭')
    #     print("* 运行在http://"+host+":"+str(port)+"/ （按CTRL+C退出）")
    #     tcp_socket=socket.socket(socket.AF_INET,socket.SOCK_STREAM)
    #     tcp_socket.bind((host,int(port)))
    #     tcp_socket.listen(1024)
    #     pack_length=1024
    #     tcp_socket.setblocking(False)
    #     tcp_socket_list=list()
    #     while True:
    #         try:
    #             new_tcp_socket,client_info=tcp_socket.accept()
    #         except:
    #             pass
    #         else:
    #             new_tcp_socket.setblocking(False)
    #             tcp_socket_list.append(new_tcp_socket)
    #         for cli_soc in tcp_socket_list:
    #             try:
    #                 data=cli_soc.recv(pack_length).decode()
    #             except Exception as e:
    #                 pass
    #             else:
    #                 if data:
    #                     datas=data.split("\r\n")
    #                     data1=datas[0]
    #                     #reqsest
    #                     REQUEST_METHOD=data1.split("/")[0].replace(' ','') ##GET
    #                     RAW_URI=re.findall(REQUEST_METHOD+"(.+?) HTTP", data1) #/aa/bb/cc?a=1&b=1
    #                     if RAW_URI:
    #                         RAW_URI=RAW_URI[0]
    #                     else:
    #                         RAW_URI=''
    #                     PATH_INFO=RAW_URI.split("?")[0] #/aa/bb/cc
    #                     QUERY_STRING=RAW_URI.replace(str(PATH_INFO),'').replace('?','') #a=1&b=1
    #                     SERVER_PROTOCOL=data1.split(" ")[-1] #HTTP/1.1
    #                     HTTP_HOST=re.findall("Host: (.+?)\r\n", data)#212.129.149.238:39010
    #                     if HTTP_HOST:
    #                         HTTP_HOST=HTTP_HOST[0]
    #                     else:
    #                         HTTP_HOST=''
    #                     HTTP_COOKIE=re.findall("Cookie: (.+?)\r\n", data)#cookie
    #                     if HTTP_COOKIE:
    #                         HTTP_COOKIE=HTTP_COOKIE[0]
    #                     else:
    #                         HTTP_COOKIE=''
    #                     REMOTE_ADDR=''
    #                     HTTP_USER_AGENT=re.findall("User-Agent: (.+?)\r\n", data) #Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.36 SE 2.X MetaSr 1.0
    #                     if HTTP_USER_AGENT:
    #                         HTTP_USER_AGENT=HTTP_USER_AGENT[0]
    #                     else:
    #                         HTTP_USER_AGENT=''
    #                     BODY_DATA=datas[len(datas)-1]
    #                     # print(data)
    #                     #reqsest
    #                     reqheader={ 
    #                         'REQUEST_METHOD':REQUEST_METHOD,
    #                         'RAW_URI':RAW_URI,
    #                         'PATH_INFO':PATH_INFO,
    #                         'QUERY_STRING':QUERY_STRING,
    #                         'SERVER_PROTOCOL':SERVER_PROTOCOL,
    #                         'HTTP_HOST':HTTP_HOST,
    #                         'HTTP_COOKIE':HTTP_COOKIE,
    #                         'REMOTE_ADDR':REMOTE_ADDR,
    #                         'HTTP_X_REAL_IP':'',
    #                         'HTTP_USER_AGENT':HTTP_USER_AGENT,
    #                         'BODY_DATA':BODY_DATA
    #                     }
    #                     p=(config.app['staticpath']+RAW_URI.replace(' ',''))
    #                     # print("目录",p)
    #                     status='200 ok'
    #                     if os.path.isfile(p):
    #                         # print('静态文件',p)
    #                         kind = filetype.guess(p)
    #                         if kind is None:
                                
    #                             f=open(p,"rb")
    #                             body=f.read()
    #                             f.close()
    #                             resheader=[("Cache-Control","public, max-age=43200"),("Expires","Thu, 07 Nov 2019 02:59:02 GMT")]
                                
    #                             header="HTTP/1.1 %s \n" % status
    #                             header+="Content-Length:%d\n" % len(body)
    #                         else:
    #                             f=open(p,"rb")
    #                             body=f.read()
    #                             f.close()
    #                             resheader=[("Content-Type",kind.mime),("Cache-Control","public, max-age=43200"),("Accept-Ranges","bytes"),("Expires","Thu, 07 Nov 2019 02:59:02 GMT")]
    #                             header="HTTP/1.1 %s \n" % status
    #                             header+="Content-Length:%d\n" % len(body)
    #                     else:
    #                         status,resheader,body=self.__routes(reqheader)
    #                         if type(body) is bytes:
    #                             pass
    #                         else:
    #                             body=body.encode()
    #                         header="HTTP/1.1 %s \n" % status
    #                         header+="Content-Length:%d\n" % len(body)
                        
    #                     print(HTTP_HOST+' -- ['+str(time.strftime("%Y-%m-%d %H:%M:%S", time.localtime()))+'] "'+REQUEST_METHOD+" "+RAW_URI +" "+SERVER_PROTOCOL + '" '+status+"-")
    #                     t=time.time()
    #                     header+="Server:kcweb"+str(config.kcweb['version'])+"\n"
    #                     header+="Date:%s\n" % datetime.utcnow().strftime('%a, %d %b %Y %H:%M:%S GMT')
    #                     for t in resheader:
    #                         header+="%s:%s\n" % (t[0],t[1])
    #                     header+="\n"
    #                     try:
    #                         cli_soc.send(header.encode())
    #                         cli_soc.send(body)
    #                     except Exception as e:
    #                         cli_soc.close()
    #                 else:
    #                     cli_soc.close()
    #                     tcp_socket_list.remove(cli_soc)
    #     tcp_socket.close()
